@@ -113,7 +113,7 @@ The array is used because the family has two boys.
 
 If a element has both of a text node and attributes
 or both of a text node and other child nodes,
-value of a text node is moved to C<'#text'> like child nodes.
+value of a text node is moved to C<#text> like child nodes.
 
     use XML::TreePP;
     use Data::Dumper;
@@ -132,24 +132,24 @@ The result dumped is following:
         }
     };
 
-The special node name of C<'#text'> is used because this elements
+The special node name of C<#text> is used because this elements
 has attribute(s) in addition to the text node.
 
 =head1 METHODS
 
 =head2 $tpp = XML::TreePP->new( %options );
 
-This constructor method returns a new XML::TreePP object.
+This constructor method returns a new XML::TreePP object with C<%options>.
 
 =head2 $tpp->set( option_name => $option_value );
 
-This method sets a option value for 'option_name'.
-If $option_value is not defined, its option is deleted.
-Options below are available:
+This method sets a option value for C<option_name>.
+If C<$option_value> is not defined, its option is deleted.
+See L</OPTIONS> section below.
 
 =head2 $tpp->get( 'option_name' );
 
-This method returns a current option value for 'option_name'.
+This method returns a current option value for C<option_name>.
 
 =head2 $tree = $tpp->parse( $source );
 
@@ -165,22 +165,27 @@ The first argument is a filename.
 
 This method receives a XML file from a remote server via HTTP and
 returns a hash tree converted.
-$method is a method of HTTP connection: GET/POST/PUT/DELETE
-$url is an URI of a XML file.
-$body is a request body when you use POST method.
-$head is a request headers as a hash ref.
+C<$method> is a method of HTTP connection: GET/POST/PUT/DELETE
+C<$url> is an URI of a XML file.
+C<$body> is a request body when you use POST method.
+C<$head> is a request headers as a hash ref.
 L<LWP::UserAgent> module or L<HTTP::Lite> module is required to fetch a file.
 
+=head2 ( $tree, $xml, $code ) = $tpp->parsehttp( $method, $url, $body, $head );
+
+In array context, This method returns also raw XML source received
+and HTTP response's status code.
+
 =head2 $source = $tpp->write( $tree, $encode );
 
 This method parses a hash tree and returns a XML source generated.
-$tree is a referecen to a hash tree.
+C<$tree> is a referecen to a hash tree.
 
 =head2 $tpp->writefile( $file, $tree, $encode );
 
 This method parses a hash tree and writes a XML source into a file.
-$file is a filename to create.
-$tree is a referecen to a hash tree.
+C<$file> is a filename to create.
+C<$tree> is a referecen to a hash tree.
 
 =head1 OPTIONS
 
@@ -188,16 +193,26 @@ $tree is a referecen to a hash tree.
 
 You can define a encoding of xml file generated by write/writefile
 methods. On Perl 5.8.x and later, you can select it from every
-encodings supported by Encode.pm. On Perl 5.6.x or before with
-Jcode.pm, you can use 'Shift_JIS', 'EUC-JP', 'ISO-2022-JP' and
-'UTF-8'. The default value is 'UTF-8'.
+encodings supported by Encode.pm. On Perl 5.6.x and before with
+Jcode.pm, you can use C<Shift_JIS>, C<EUC-JP>, C<ISO-2022-JP> and
+C<UTF-8>. The default value is C<UTF-8>.
 
 =head2 $tpp->set( force_array => [ 'rdf:li', 'item', '-xmlns' ] );
 
 This option allows you to specify a list of element names which
-should always be forced into an array representation
+should always be forced into an array representation.
 The default value is null, it means that context of the elements
-will determine to make array or to keep it scalar.
+will determine to make array or to keep it scalar or hash.
+Note that the special wildcard name C<'*'> means all elements.
+
+=head2 $tpp->set( force_hash => [ 'item', 'image' ] );
+
+This option allows you to specify a list of element names which
+should always be forced into an hash representation.
+The default value is null, it means that context of the elements
+will determine to make hash or to keep it scalar as a text node.
+See also C<text_node_key> option below.
+Note that the special wildcard name C<'*'> means all elements.
 
 =head2 $tpp->set( first_out => [ 'link', 'title', '-type' ] );
 
@@ -220,7 +235,7 @@ default, cdata section is converted into a scalar.
 
 This option allows you to specify a HTTP_USER_AGENT string which
 is used by parsehttp() method.
-The default string is C<"XML-TreePP/#.##">, where C<"#.##"> is
+The default string is C<'XML-TreePP/#.##'>, where C<'#.##'> is
 substituted with the version number of this library.
 
 =head2 $tpp->set( attr_prefix => '@' );
@@ -234,7 +249,7 @@ Zero-length prefix C<''> is also available now.
 =head2 $tpp->set( text_node_key => '#text' );
 
 This option allows you to specify a hash key for text nodes.
-The default key is C<'#text'>.
+The default key is C<#text>.
 
 =head2 $tpp->set( ignore_error => 1 );
 
@@ -249,12 +264,12 @@ This option forces to change or leave it.
 =head2 $tpp->set( http_lite => $http );
 
 This option forces pasrsehttp() method to use L<HTTP::Lite> module 
-with its instance created like: C<$http = HTTP::Lite->new();>
+with its instance created like: C<$http = HTTP::Lite-E<gt>new();>
 
 =head2 $tpp->set( lwp_useragent => $ua );
 
 This option forces pasrsehttp() method to use L<LWP::UserAgent> module 
-with its instance created like: C<$ua = LWP::UserAgent->new();>
+with its instance created like: C<$ua = LWP::UserAgent-E<gt>new();>
 
 =head2 $tpp->set( use_ixhash => 1 );
 
@@ -272,10 +287,24 @@ This doesn't strictly follow the XML Document Spec but does looks nice.
 This makes utf8 flag on for every element's value parsed
 and makes it on for an XML code generated as well.
 
-=head2 $tpp->set( base_class => 'class' );
+=head2 $tpp->set( base_class => 'MyElement' );
 
-This declares (blesses) class name and its subclass names for each element.
-You may use this with L<Class::Accessor>.
+This declares (blesses) class name for each element's hashref.
+Each class is named straight as a child class of it parent class.
+
+    <root><parent><child>text</child></parent></root>
+
+E.g., <child> element above is blessed to C<MyElement::root::parent::child> 
+class. You may use this with L<MyElement::Accessor>.
+
+=head2 $tpp->set( elem_class => 'MyElement' );
+
+This declares (blesses) class name for each element's hashref.
+Each class is named horizontally under the direct child of C<MyElement>.
+
+    <root><parent><child>text</child></parent></root>
+
+E.g., <child> element above is blessed to C<MyElement::child> class.
 
 =head1 AUTHOR
 
@@ -295,7 +324,7 @@ use Carp;
 use Symbol;
 
 use vars qw( $VERSION );
-$VERSION = '0.22';
+$VERSION = '0.24';
 
 my $XML_ENCODING      = 'UTF-8';
 my $INTERNAL_ENCODING = 'UTF-8';
@@ -517,9 +546,10 @@ sub parsehttp_lwp {
     }
     $req->content($body) if defined $body;
     my $res = $ua->request($req);
-    return unless $res->is_success();
+    my $code = $res->code();
     my $text = $res->content();
-    $self->parse( \$text );
+    my $tree = $self->parse( \$text ) if $res->is_success();
+    wantarray ? ( $tree, $text, $code ) : $tree;
 }
 
 sub parsehttp_lite {
@@ -543,9 +573,10 @@ sub parsehttp_lite {
         $http->add_req_header( 'User-Agent', $self->{__user_agent} );
     }
     $http->{content} = $body if defined $body;
-    $http->request($url) or return;
+    my $code = $http->request($url) or return;
     my $text = $http->body();
-    $self->parse( \$text );
+    my $tree = $self->parse( \$text );
+    wantarray ? ( $tree, $text, $code ) : $tree;
 }
 
 sub parsefile {
@@ -583,10 +614,21 @@ sub parse {
     }
 
     local $self->{__force_array};
+    local $self->{__force_array_all};
     if ( exists $self->{force_array} ) {
         my $force = $self->{force_array};
         $force = [$force] unless ref $force;
         $self->{__force_array} = { map { $_ => 1 } @$force };
+        $self->{__force_array_all} = $self->{__force_array}->{'*'};
+    }
+
+    local $self->{__force_hash};
+    local $self->{__force_hash_all};
+    if ( exists $self->{force_hash} ) {
+        my $force = $self->{force_hash};
+        $force = [$force] unless ref $force;
+        $self->{__force_hash} = { map { $_ => 1 } @$force };
+        $self->{__force_hash_all} = $self->{__force_hash}->{'*'};
     }
 
     my $tnk = $self->{text_node_key} if exists $self->{text_node_key};
@@ -601,9 +643,16 @@ sub parse {
         return $self->die( "Tie::IxHash is required." ) unless &load_tie_ixhash();
     }
 
-    my $flat = $self->xml_to_flat($textref);
-    my $base = $self->{base_class} if exists $self->{base_class};
-    my $tree = $self->flat_to_tree( $flat, '', $base );
+    my $flat  = $self->xml_to_flat($textref);
+    my $class = $self->{base_class} if exists $self->{base_class};
+    my $tree  = $self->flat_to_tree( $flat, '', $class );
+    if ( defined $class ) {
+        bless( $tree, $class );
+    }
+    elsif ( exists $self->{elem_class} && $self->{elem_class} ) {
+        bless( $tree, $self->{elem_class} );
+    } 
+
     wantarray ? ( $tree, $$textref ) : $tree;
 }
 
@@ -645,7 +694,7 @@ sub xml_to_flat {
                 $node->{endTag}++;
             }
             elsif ( $contElem =~ s#/$## ) {
-                # one line
+                $node->{emptyTag}++;
             }
             else {
                 $node->{startTag}++;
@@ -717,55 +766,81 @@ sub flat_to_tree {
             return $self->die( "Invalid tag sequence: <$parent></$name>" );
         }
         my $elem = $node->{attributes};
+        my $forcehash = $self->{__force_hash_all} || $self->{__force_hash}->{$name};
         if ( $node->{startTag} ) {              # recursive call
             my $subclass;
             if ( defined $class ) {
-                $subclass = $name;
-                $subclass =~ s/\W/_/sg;
-                $subclass = $class.'::'.$subclass;
+                my $escname = $name;
+                $escname =~ s/\W/_/sg;
+                $subclass = $class.'::'.$escname;
             }
             my $child = $self->flat_to_tree( $source, $name, $subclass );
-            if ( ref $elem && scalar keys %$elem ) {
-                if ( UNIVERSAL::isa( $child, "HASH" ) ) {
+            my $hasattr = scalar keys %$elem if ref $elem;
+            if ( UNIVERSAL::isa( $child, "HASH" ) ) {
+                if ( $hasattr ) {
                     # some attributes and some child nodes
-                    foreach my $key ( keys %$child ) {
-                        $elem->{$key} = $child->{$key};
-                    }
+                    %$elem = ( %$elem, %$child );
                 }
-                elsif ( defined $child ) {
+                else {
+                    # some child nodes without attributes
+                    $elem = $child;
+                }
+            }
+            elsif ( defined $child ) {
+                if ( $hasattr ) {
                     # some attributes and text node
                     $elem->{$self->{text_node_key}} = $child;
                 }
+                elsif ( $forcehash ) {
+                    # only text node without attributes
+                    $elem = { $self->{text_node_key} => $child };
+                }
+                else {
+                    # text node without attributes
+                    $elem = $child;
+                }
             }
-            else {
-                # no attributes and text node or nothing
-                $elem = $child;
-            }
-            if ( $subclass && UNIVERSAL::isa( $elem, "HASH" ) ) {
-                bless( $elem, $subclass );
+            if ( UNIVERSAL::isa( $elem, "HASH" ) ) {
+                if ( defined $subclass ) {
+                    bless( $elem, $subclass );
+                } elsif ( exists $self->{elem_class} && $self->{elem_class} ) {
+                    my $escname = $name;
+                    $escname =~ s/\W/_/sg;
+                    my $elmclass = $self->{elem_class}.'::'.$escname;
+                    bless( $elem, $elmclass );
+                } 
             }
         }
+        elsif ( $forcehash && ! ref $elem ) {
+            $elem = {};
+        }
         # next unless defined $elem;
         $tree->{$name} ||= [];
         push( @{ $tree->{$name} }, $elem );
     }
-    foreach my $key ( keys %$tree ) {
-        next if $self->{__force_array}->{$key};
-        next if ( 1 < scalar @{ $tree->{$key} } );
-        $tree->{$key} = shift @{ $tree->{$key} };
+    if ( ! $self->{__force_array_all} ) {
+        foreach my $key ( keys %$tree ) {
+            next if $self->{__force_array}->{$key};
+            next if ( 1 < scalar @{ $tree->{$key} } );
+            $tree->{$key} = shift @{ $tree->{$key} };
+        }
     }
+    my $haschild = scalar keys %$tree;
     if ( scalar @$text ) {
         if ( scalar @$text == 1 ) {
+            # one text node (normal)
             $text = shift @$text;
         }
         elsif ( ! scalar grep {ref $_} @$text ) {
+            # some text node splitted
             $text = join( '', @$text );
         }
         else {
+            # some cdata node
             my $join = join( '', map {ref $_ ? $$_ : $_} @$text );
             $text = \$join;
         }
-        if ( scalar keys %$tree ) {
+        if ( $haschild ) {
             # some child nodes and also text node
             $tree->{$self->{text_node_key}} = $text;
         }
@@ -774,6 +849,10 @@ sub flat_to_tree {
             $tree = $text;
         }
     }
+    elsif ( ! $haschild ) {
+        # no child and no text
+        $tree = "";
+    }
     $tree;
 }
 
